@@ -30,9 +30,9 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n-lib.h>
 
-#ifdef HAVE_MOBLIN
-#include <nbtk/nbtk-gtk.h>
-#endif
+#ifdef HAVE_MEEGO
+#include <mx/mx-gtk.h>
+#endif /* HAVE_MEEGO */
 
 #include <libempathy/empathy-utils.h>
 
@@ -745,13 +745,13 @@ account_widget_applied_cb (GObject *source_object,
           gboolean enabled_checked;
 
           enabled_checked =
-#ifdef HAVE_MOBLIN
-            nbtk_gtk_light_switch_get_active (
-                NBTK_GTK_LIGHT_SWITCH (priv->enabled_checkbox));
+#ifdef HAVE_MEEGO
+            mx_gtk_light_switch_get_active (
+                MX_GTK_LIGHT_SWITCH (priv->enabled_checkbox));
 #else
             gtk_toggle_button_get_active (
                 GTK_TOGGLE_BUTTON (priv->enabled_checkbox));
-#endif
+#endif /* HAVE_MEEGO */
 
           if (tp_account_is_enabled (account) && enabled_checked)
             {
@@ -940,6 +940,78 @@ account_widget_is_gtalk (EmpathyAccountWidget *self)
       "im-google-talk");
 }
 
+static gboolean
+account_widget_is_facebook (EmpathyAccountWidget *self)
+{
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+
+  return !tp_strdiff (empathy_account_settings_get_icon_name (priv->settings),
+      "im-facebook");
+}
+
+#define FACEBOOK_SUFFIX "@chat.facebook.com"
+
+static void
+facebook_id_widget_changed_cb (GtkWidget *entry,
+    EmpathyAccountWidget *self)
+{
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+  const gchar *account;
+
+  account_widget_entry_changed_common (self, GTK_ENTRY (entry), FALSE);
+
+  account = empathy_account_settings_get_string (priv->settings, "account");
+  if (!EMP_STR_EMPTY (account) &&
+      !g_str_has_suffix (account, FACEBOOK_SUFFIX))
+    {
+      gchar *tmp;
+
+      tmp = g_strdup_printf ("%s%s", account, FACEBOOK_SUFFIX);
+
+      DEBUG ("Change account from '%s' to '%s'", account, tmp);
+
+      empathy_account_settings_set_string (priv->settings, "account", tmp);
+      g_free (tmp);
+    }
+
+  empathy_account_widget_changed (self);
+}
+
+static gchar *
+remove_facebook_suffix (const gchar *str)
+{
+  if (!g_str_has_suffix (str, FACEBOOK_SUFFIX))
+    return g_strdup (str);
+
+  return g_strndup (str, strlen (str) - strlen (FACEBOOK_SUFFIX));
+}
+
+static void
+setup_facebook_id_widget (EmpathyAccountWidget *self,
+    GtkWidget *widget)
+{
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+  const gchar *str = NULL;
+
+  g_object_set_data_full (G_OBJECT (widget), "param_name",
+      g_strdup ("account"), g_free);
+
+  str = empathy_account_settings_get_string (priv->settings, "account");
+  if (str != NULL)
+    {
+      gchar *tmp;
+
+      tmp = remove_facebook_suffix (str);
+      gtk_entry_set_text (GTK_ENTRY (widget), tmp);
+      g_free (tmp);
+    }
+
+  priv->param_account_widget = widget;
+
+  g_signal_connect (widget, "changed",
+      G_CALLBACK (facebook_id_widget_changed_cb), self);
+}
+
 static void
 account_widget_build_jabber (EmpathyAccountWidget *self,
     const char *filename)
@@ -949,13 +1021,17 @@ account_widget_build_jabber (EmpathyAccountWidget *self,
   GtkWidget *checkbutton_ssl;
   GtkWidget *label_id, *label_password;
   GtkWidget *label_id_create, *label_password_create;
-  GtkWidget *label_example_gtalk, *label_example_jabber;
-  gboolean is_gtalk;
+  GtkWidget *label_example_gtalk, *label_example_jabber, *label_example_fb;
+  gboolean is_gtalk, is_facebook;
+  GtkWidget *expander_advanced;
+  GtkWidget *entry_id;
 
   is_gtalk = account_widget_is_gtalk (self);
+  is_facebook = account_widget_is_facebook (self);
 
-  if (priv->simple && !is_gtalk)
+  if (priv->simple && !is_gtalk && !is_facebook)
     {
+      /* Simple widget for XMPP */
       self->ui_details->gui = empathy_builder_get_file (filename,
           "vbox_jabber_simple", &self->ui_details->widget,
           "label_id_simple", &label_id,
@@ -981,6 +1057,7 @@ account_widget_build_jabber (EmpathyAccountWidget *self,
     }
   else if (priv->simple && is_gtalk)
     {
+      /* Simple widget for Google Talk */
       self->ui_details->gui = empathy_builder_get_file (filename,
           "vbox_gtalk_simple", &self->ui_details->widget,
           NULL);
@@ -992,8 +1069,25 @@ account_widget_build_jabber (EmpathyAccountWidget *self,
 
       self->ui_details->default_focus = g_strdup ("entry_id_g_simple");
     }
+  else if (priv->simple && is_facebook)
+    {
+      /* Simple widget for Facebook */
+      self->ui_details->gui = empathy_builder_get_file (filename,
+          "vbox_fb_simple", &self->ui_details->widget,
+          "entry_id_fb_simple", &entry_id,
+          NULL);
+
+      empathy_account_widget_handle_params (self,
+          "entry_password_fb_simple", "password",
+          NULL);
+
+      setup_facebook_id_widget (self, entry_id);
+
+      self->ui_details->default_focus = g_strdup ("entry_id_fb_simple");
+    }
   else
     {
+      /* Full widget for XMPP, Google Talk and Facebook*/
       self->ui_details->gui = empathy_builder_get_file (filename,
           "table_common_settings", &priv->table_common_settings,
           "vbox_jabber_settings", &self->ui_details->widget,
@@ -1001,10 +1095,13 @@ account_widget_build_jabber (EmpathyAccountWidget *self,
           "checkbutton_ssl", &checkbutton_ssl,
           "label_username_example", &label_example_jabber,
           "label_username_g_example", &label_example_gtalk,
+          "label_username_f_example", &label_example_fb,
+          "expander_advanced", &expander_advanced,
+          "entry_id", &entry_id,
+          "label_id", &label_id,
           NULL);
 
       empathy_account_widget_handle_params (self,
-          "entry_id", "account",
           "entry_password", "password",
           "entry_resource", "resource",
           "entry_server", "server",
@@ -1015,6 +1112,19 @@ account_widget_build_jabber (EmpathyAccountWidget *self,
           "checkbutton_encryption", "require-encryption",
           NULL);
 
+      if (is_facebook)
+        {
+          gtk_label_set_label (GTK_LABEL (label_id), _("Username:"));
+
+          /* Facebook special case the entry ID widget to hide the
+           * "@chat.facebook.com" part */
+          setup_facebook_id_widget (self, entry_id);
+        }
+      else
+        {
+          empathy_account_widget_setup_widget (self, entry_id, "account");
+        }
+
       self->ui_details->default_focus = g_strdup ("entry_id");
       self->ui_details->add_forget = TRUE;
       priv->spinbutton_port = spinbutton_port;
@@ -1028,6 +1138,12 @@ account_widget_build_jabber (EmpathyAccountWidget *self,
           gtk_widget_hide (label_example_jabber);
           gtk_widget_show (label_example_gtalk);
         }
+      else if (is_facebook)
+        {
+          gtk_widget_hide (label_example_jabber);
+          gtk_widget_show (label_example_fb);
+          gtk_widget_hide (expander_advanced);
+        }
     }
 }
 
@@ -1214,34 +1330,34 @@ empathy_account_widget_enabled_cb (TpAccount *account,
 
   if (priv->enabled_checkbox != NULL)
     {
-#ifdef HAVE_MOBLIN
-      nbtk_gtk_light_switch_set_active (
-          NBTK_GTK_LIGHT_SWITCH (priv->enabled_checkbox),
+#ifdef HAVE_MEEGO
+      mx_gtk_light_switch_set_active (
+          MX_GTK_LIGHT_SWITCH (priv->enabled_checkbox),
           enabled);
 #else
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->enabled_checkbox),
           enabled);
-#endif /* HAVE_MOBLIN */
+#endif /* HAVE_MEEGO */
     }
 }
 
 static void
-#ifdef HAVE_MOBLIN
-account_widget_switch_flipped_cb (NbtkGtkLightSwitch *sw,
+#ifdef HAVE_MEEGO
+account_widget_switch_flipped_cb (MxGtkLightSwitch *sw,
     gboolean state,
     gpointer user_data)
 #else
 account_widget_enabled_released_cb (GtkToggleButton *toggle_button,
     gpointer user_data)
-#endif /* HAVE_MOBLIN */
+#endif /* HAVE_MEEGO */
 {
   EmpathyAccountWidgetPriv *priv = GET_PRIV (user_data);
   TpAccount *account;
-#ifndef HAVE_MOBLIN
+#ifndef HAVE_MEEGO
   gboolean state;
 
   state = gtk_toggle_button_get_active (toggle_button);
-#endif
+#endif /* HAVE_MEEGO */
 
   account = empathy_account_settings_get_account (priv->settings);
 
@@ -1320,9 +1436,10 @@ presence_changed_cb (TpAccountManager *manager,
     /* This button doesn't exist in 'simple' mode */
     return;
 
-  if (state > TP_CONNECTION_PRESENCE_TYPE_OFFLINE)
+  if (state > TP_CONNECTION_PRESENCE_TYPE_OFFLINE &&
+      priv->creating_account)
     {
-      /* We are online, display a Login button */
+      /* We are online and creating a new account, display a Login button */
       GtkWidget *image;
 
       gtk_button_set_use_stock (GTK_BUTTON (priv->apply_button), FALSE);
@@ -1334,10 +1451,11 @@ presence_changed_cb (TpAccountManager *manager,
     }
   else
     {
-      /* We are offline, display a Save button */
+      /* We are offline or modifying an existing account, display
+       * a Save button */
       gtk_button_set_image (GTK_BUTTON (priv->apply_button), NULL);
       gtk_button_set_use_stock (GTK_BUTTON (priv->apply_button), TRUE);
-      gtk_button_set_label (GTK_BUTTON (priv->apply_button), GTK_STOCK_SAVE);
+      gtk_button_set_label (GTK_BUTTON (priv->apply_button), GTK_STOCK_APPLY);
     }
 }
 
@@ -1378,11 +1496,11 @@ add_enable_checkbox (EmpathyAccountWidget *self,
     TpAccount *account)
 {
   EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
-#ifdef HAVE_MOBLIN
+#ifdef HAVE_MEEGO
   GtkWidget *w;
 #else
   GtkWidget *vbox = self->ui_details->widget;
-#endif
+#endif /* HAVE_MEEGO */
   guint nb_rows, nb_columns;
   gboolean is_enabled;
 
@@ -1392,14 +1510,14 @@ add_enable_checkbox (EmpathyAccountWidget *self,
 
   is_enabled = tp_account_is_enabled (account);
 
-#ifdef HAVE_MOBLIN
+#ifdef HAVE_MEEGO
   w = gtk_label_new (_("Account:"));
   gtk_misc_set_alignment (GTK_MISC (w), 0, 0.5);
 
-  priv->enabled_checkbox = nbtk_gtk_light_switch_new ();
+  priv->enabled_checkbox = mx_gtk_light_switch_new ();
 
-  nbtk_gtk_light_switch_set_active (
-      NBTK_GTK_LIGHT_SWITCH (priv->enabled_checkbox), is_enabled);
+  mx_gtk_light_switch_set_active (
+      MX_GTK_LIGHT_SWITCH (priv->enabled_checkbox), is_enabled);
 
   gtk_widget_show (w);
 #else
@@ -1408,7 +1526,7 @@ add_enable_checkbox (EmpathyAccountWidget *self,
 
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->enabled_checkbox),
       is_enabled);
-#endif /* HAVE_MOBLIN */
+#endif /* HAVE_MEEGO */
 
   g_object_get (priv->table_common_settings, "n-rows", &nb_rows,
       "n-columns", &nb_columns, NULL);
@@ -1416,7 +1534,7 @@ add_enable_checkbox (EmpathyAccountWidget *self,
   gtk_table_resize (GTK_TABLE (priv->table_common_settings), ++nb_rows,
       nb_columns);
 
-#ifdef HAVE_MOBLIN
+#ifdef HAVE_MEEGO
   gtk_table_attach (GTK_TABLE (priv->table_common_settings),
       w,
       0, 1, nb_rows - 1, nb_rows,
@@ -1428,21 +1546,21 @@ add_enable_checkbox (EmpathyAccountWidget *self,
 #else
   gtk_box_pack_start (GTK_BOX (vbox), priv->enabled_checkbox, FALSE, FALSE, 0);
   gtk_box_reorder_child (GTK_BOX (vbox), priv->enabled_checkbox, 0);
-#endif /* HAVE_MOBLIN */
+#endif /* HAVE_MEEGO */
 
   gtk_widget_show (priv->enabled_checkbox);
 
-#ifdef HAVE_MOBLIN
+#ifdef HAVE_MEEGO
   g_signal_connect (G_OBJECT (priv->enabled_checkbox), "switch-flipped",
       G_CALLBACK (account_widget_switch_flipped_cb), self);
 #else
   g_signal_connect (G_OBJECT (priv->enabled_checkbox), "released",
       G_CALLBACK (account_widget_enabled_released_cb), self);
-#endif /* HAVE_MOBLIN */
+#endif /* HAVE_MEEGO */
 }
 
-#ifndef HAVE_MOBLIN
-/* Moblin doesn't support registration */
+#ifndef HAVE_MEEGO
+/* Meego doesn't support registration */
 static void
 add_register_buttons (EmpathyAccountWidget *self,
     TpAccount *account)
@@ -1462,7 +1580,7 @@ add_register_buttons (EmpathyAccountWidget *self,
   if (!tp_connection_manager_protocol_can_register (protocol))
     return;
 
-  if (account_widget_is_gtalk (self))
+  if (account_widget_is_gtalk (self) || account_widget_is_facebook (self))
     return;
 
   if (priv->simple)
@@ -1481,7 +1599,7 @@ add_register_buttons (EmpathyAccountWidget *self,
   gtk_widget_show (priv->radiobutton_reuse);
   gtk_widget_show (radiobutton_register);
 }
-#endif
+#endif /* HAVE_MEEGO */
 
 static void
 do_constructed (GObject *obj)
@@ -1585,21 +1703,13 @@ do_constructed (GObject *obj)
 
       priv->cancel_button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
 
-      if (priv->creating_account)
-        {
-          /* Assumre we are offline, display a Save button. We'll update
-           * it once the account manager is ready if needed */
-          priv->apply_button = gtk_button_new_from_stock (GTK_STOCK_SAVE);
+      priv->apply_button = gtk_button_new_from_stock (GTK_STOCK_APPLY);
 
-          empathy_signal_connect_weak (priv->account_manager,
-              "most-available-presence-changed",
-              G_CALLBACK (presence_changed_cb), obj);
-        }
-      else
-        {
-          /* We are editing an existing account, display an Apply button */
-          priv->apply_button = gtk_button_new_from_stock (GTK_STOCK_APPLY);
-        }
+      /* We'll change this button to a "Log in" one if we are creating a new
+       * account and are connected. */
+      empathy_signal_connect_weak (priv->account_manager,
+          "most-available-presence-changed",
+          G_CALLBACK (presence_changed_cb), obj);
 
       gtk_box_pack_end (GTK_BOX (hbox), priv->apply_button, TRUE,
           TRUE, 3);
@@ -1635,9 +1745,9 @@ do_constructed (GObject *obj)
           G_CALLBACK (empathy_account_widget_enabled_cb), self);
     }
 
-#ifndef HAVE_MOBLIN
+#ifndef HAVE_MEEGO
   add_register_buttons (self, account);
-#endif
+#endif /* HAVE_MEEGO */
   add_enable_checkbox (self, account);
 
   /* hook up to widget destruction to unref ourselves */
@@ -1858,6 +1968,14 @@ empathy_account_widget_get_default_display_name (EmpathyAccountWidget *self)
           default_display_name = g_strdup_printf (_("%1$s on %2$s"),
               login_id, server);
         }
+      else if (account_widget_is_facebook (self))
+        {
+          gchar *tmp;
+
+          tmp = remove_facebook_suffix (login_id);
+          default_display_name = g_strdup_printf ("Facebook (%s)", tmp);
+          g_free (tmp);
+        }
       else
         {
           default_display_name = g_strdup (login_id);
