@@ -683,6 +683,8 @@ contact_list_view_popup_menu_idle_cb (gpointer user_data)
 		gtk_menu_popup (GTK_MENU (menu),
 				NULL, NULL, NULL, NULL,
 				data->button, data->time);
+		g_object_ref_sink (menu);
+		g_object_unref (menu);
 	}
 
 	g_slice_free (MenuPopupData, data);
@@ -795,6 +797,8 @@ contact_list_view_call_activated_cb (
 	gtk_widget_show (menu);
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
 			event->button, event->time);
+	g_object_ref_sink (menu);
+	g_object_unref (menu);
 
 	g_object_unref (contact);
 }
