@@ -826,9 +826,10 @@ popup_avatar_menu (EmpathyContactWidget *information,
       event_time = gtk_get_current_event_time ();
     }
 
-  gtk_menu_attach_to_widget (GTK_MENU (menu), parent, NULL);
   gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
       button, event_time);
+  g_object_ref_sink (menu);
+  g_object_unref (menu);
 }
 
 static gboolean
