@@ -407,12 +407,14 @@ log_window_find_changed_cb (GtkTreeSelection *selection,
 
 	/* Highlight and find messages */
 	empathy_chat_view_highlight (window->chatview_find,
-				    window->last_find);
+				    window->last_find, FALSE);
 	empathy_chat_view_find_next (window->chatview_find,
 				    window->last_find,
-				    TRUE);
+				    TRUE,
+				    FALSE);
 	empathy_chat_view_find_abilities (window->chatview_find,
 					 window->last_find,
+					 FALSE,
 					 &can_do_previous,
 					 &can_do_next);
 	gtk_widget_set_sensitive (window->button_previous, can_do_previous);
@@ -605,9 +607,11 @@ log_window_button_next_clicked_cb (GtkWidget       *widget,
 
 		empathy_chat_view_find_next (window->chatview_find,
 					    window->last_find,
+					    FALSE,
 					    FALSE);
 		empathy_chat_view_find_abilities (window->chatview_find,
 						 window->last_find,
+						 FALSE,
 						 &can_do_previous,
 						 &can_do_next);
 		gtk_widget_set_sensitive (window->button_previous, can_do_previous);
@@ -625,9 +629,11 @@ log_window_button_previous_clicked_cb (GtkWidget       *widget,
 
 		empathy_chat_view_find_previous (window->chatview_find,
 						window->last_find,
+						FALSE,
 						FALSE);
 		empathy_chat_view_find_abilities (window->chatview_find,
 						 window->last_find,
+						 FALSE,
 						 &can_do_previous,
 						 &can_do_next);
 		gtk_widget_set_sensitive (window->button_previous, can_do_previous);
@@ -1108,12 +1114,13 @@ log_window_entry_chats_changed_cb (GtkWidget       *entry,
 	const gchar *str;
 
 	str = gtk_entry_get_text (GTK_ENTRY (window->entry_chats));
-	empathy_chat_view_highlight (window->chatview_chats, str);
+	empathy_chat_view_highlight (window->chatview_chats, str, FALSE);
 
 	if (str) {
 		empathy_chat_view_find_next (window->chatview_chats,
 					    str,
-					    TRUE);
+					    TRUE,
+					    FALSE);
 	}
 }
 
@@ -1128,6 +1135,7 @@ log_window_entry_chats_activate_cb (GtkWidget       *entry,
 	if (str) {
 		empathy_chat_view_find_next (window->chatview_chats,
 					    str,
+					    FALSE,
 					    FALSE);
 	}
 }
