@@ -40,6 +40,7 @@
 
 #include "empathy-ui-utils.h"
 #include "empathy-images.h"
+#include "empathy-smiley-manager.h"
 #include "empathy-conf.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_OTHER
@@ -49,13 +50,15 @@
 #include <libempathy/empathy-idle.h>
 #include <libempathy/empathy-ft-factory.h>
 
-#define SCHEMES "(https?|s?ftps?|nntp|news|javascript|about|ghelp|apt|telnet|"\
-		"file|webcal|mailto)"
-#define BODY "([^\\ \\n]+)"
-#define END_BODY "([^\\ \\n]*[^,;\?><()\\ \"\\.\\n])"
-#define URI_REGEX "("SCHEMES"://"END_BODY")" \
-		  "|((mailto:)?"BODY"@"BODY"\\."END_BODY")"\
-		  "|((www|ftp)\\."END_BODY")"
+#define SCHEMES           "([a-zA-Z\\+]+)"
+#define INVALID_CHARS     "\\s\"'"
+#define INVALID_CHARS_EXT INVALID_CHARS "\\[\\]<>(){},;:?"
+#define BODY              "([^"INVALID_CHARS"]+)"
+#define BODY_END          "([^"INVALID_CHARS"]*)[^"INVALID_CHARS_EXT".]"
+#define BODY_STRICT       "([^"INVALID_CHARS_EXT"]+)"
+#define URI_REGEX         "("SCHEMES"://"BODY_END")" \
+		          "|((www|ftp)\\."BODY_END")" \
+		          "|((mailto:)?"BODY_STRICT"@"BODY"\\."BODY_END")"
 
 void
 empathy_gtk_init (void)
@@ -227,6 +230,21 @@ empathy_icon_name_for_contact (EmpathyContact *contact)
 	return empathy_icon_name_for_presence (presence);
 }
 
+const gchar *
+empathy_protocol_name_for_contact (EmpathyContact   *contact)
+{
+	TpAccount     *account;
+
+	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
+
+	account = empathy_contact_get_account (contact);
+	if (account == NULL) {
+		return NULL;
+	}
+
+	return tp_account_get_icon_name (account);
+}
+
 GdkPixbuf *
 empathy_pixbuf_from_data (gchar *data,
 			  gsize  data_size)
@@ -487,6 +505,104 @@ empathy_pixbuf_avatar_from_contact_scaled (EmpathyContact *contact,
 	return empathy_pixbuf_from_avatar_scaled (avatar, width, height);
 }
 
+GdkPixbuf *
+empathy_pixbuf_contact_status_icon (EmpathyContact *contact,
+				   gboolean       show_protocol)
+{
+	const gchar *icon_name;
+
+	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
+
+	icon_name = empathy_icon_name_for_contact (contact);
+
+	if (icon_name == NULL) {
+		return NULL;
+	}
+	return empathy_pixbuf_contact_status_icon_with_icon_name (contact,
+	    icon_name,
+	    show_protocol);
+}
+
+GdkPixbuf *
+empathy_pixbuf_contact_status_icon_with_icon_name (EmpathyContact *contact,
+					  const gchar    *icon_name,
+					  gboolean       show_protocol)
+{
+	GdkPixbuf *pix_status;
+	GdkPixbuf *pix_protocol;
+	gchar     *icon_filename;
+	gint       height, width;
+	gint       numerator, denominator;
+
+	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
+	g_return_val_if_fail (icon_name != NULL, NULL);
+
+	numerator = 3;
+	denominator = 4;
+
+	icon_filename = empathy_filename_from_icon_name (icon_name,
+							 GTK_ICON_SIZE_MENU);
+	if (icon_filename == NULL) {
+		DEBUG ("icon name: %s could not be found\n", icon_name);
+		return NULL;
+	}
+
+	pix_status = gdk_pixbuf_new_from_file (icon_filename, NULL);
+
+	g_free (icon_filename);
+
+	if (pix_status == NULL) {
+		DEBUG ("Could not open icon %s\n", icon_filename);
+		return NULL;
+	}
+
+	if (!show_protocol)
+		return pix_status;
+
+	height = gdk_pixbuf_get_height (pix_status);
+	width = gdk_pixbuf_get_width (pix_status);
+
+	pix_protocol = empathy_pixbuf_protocol_from_contact_scaled (contact,
+								    width * numerator / denominator,
+								    height * numerator / denominator);
+
+	if (pix_protocol == NULL) {
+		return pix_status;
+	}
+	gdk_pixbuf_composite (pix_protocol, pix_status,
+	    0, height - height * numerator / denominator,
+	    width * numerator / denominator, height * numerator / denominator,
+	    0, height - height * numerator / denominator,
+	    1, 1,
+	    GDK_INTERP_BILINEAR, 255);
+
+	g_object_unref (pix_protocol);
+
+	return pix_status;
+}
+
+GdkPixbuf *
+empathy_pixbuf_protocol_from_contact_scaled (EmpathyContact *contact,
+					  gint           width,
+					  gint           height)
+{
+	TpAccount *account;
+	gchar     *filename;
+	GdkPixbuf *pixbuf = NULL;
+
+	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
+
+	account = empathy_contact_get_account (contact);
+	filename = empathy_filename_from_icon_name (tp_account_get_icon_name (account),
+						    GTK_ICON_SIZE_MENU);
+	if (filename != NULL) {
+		pixbuf = gdk_pixbuf_new_from_file_at_size (filename, width, height, NULL);
+		g_free (filename);
+	}
+
+	return pixbuf;
+}
+
 GdkPixbuf *
 empathy_pixbuf_scale_down_if_necessary (GdkPixbuf *pixbuf, gint max_size)
 {
@@ -1302,14 +1418,24 @@ empathy_window_present (GtkWindow *window,
 	/* Move the window to the current workspace before trying to show it.
 	 * This is the behaviour people expect when clicking on the statusbar icon. */
 	gdk_window = gtk_widget_get_window (GTK_WIDGET (window));
-	if (gdk_window)
+	if (gdk_window) {
+		gint x, y;
+		gint w, h;
+
+		/* Has no effect if the WM has viewports, like compiz */
 		gdk_x11_window_move_to_current_desktop (gdk_window);
 
+		/* If window is still off-screen, hide it to force it to
+		 * reposition on the current workspace. */
+		gtk_window_get_position (window, &x, &y);
+		gtk_window_get_size (window, &w, &h);
+		if (!EMPATHY_RECT_IS_ON_SCREEN (x, y, w, h))
+			gtk_widget_hide (GTK_WIDGET (window));
+	}
+
 	timestamp = gtk_get_current_event_time ();
 	gtk_window_present_with_time (window, timestamp);
 	gtk_window_set_skip_taskbar_hint (window, FALSE);
-	/* FIXME: This shouldn't be required as gtk_window_present's doc says
-	 *        it deiconify automatically. */
 	gtk_window_deiconify (window);
 }
 
@@ -1322,7 +1448,7 @@ empathy_get_toplevel_window (GtkWidget *widget)
 
 	toplevel = gtk_widget_get_toplevel (widget);
 	if (GTK_IS_WINDOW (toplevel) &&
-	    GTK_WIDGET_TOPLEVEL (toplevel)) {
+	    gtk_widget_is_toplevel (toplevel)) {
 		return GTK_WINDOW (toplevel);
 	}
 
@@ -1441,30 +1567,70 @@ empathy_toggle_button_set_state_quietly (GtkWidget *widget,
 	g_signal_handlers_unblock_by_func (widget, callback, user_data);
 }
 
+void
+empathy_send_file (EmpathyContact *contact, GFile *file)
+{
+	EmpathyFTFactory *factory;
+	GtkRecentManager *manager;
+	gchar *uri;
+
+	g_return_if_fail (EMPATHY_IS_CONTACT (contact));
+	g_return_if_fail (G_IS_FILE (file));
+
+	factory = empathy_ft_factory_dup_singleton ();
+
+	empathy_ft_factory_new_transfer_outgoing (factory, contact, file);
+
+	uri = g_file_get_uri (file);
+	manager = gtk_recent_manager_get_default ();
+	gtk_recent_manager_add_item (manager, uri);
+	g_free (uri);
+
+	g_object_unref (factory);
+}
+
+void
+empathy_send_file_from_uri_list (EmpathyContact *contact, const gchar *uri_list)
+{
+	const gchar *nl;
+	GFile *file;
+
+	/* Only handle a single file for now.  It would be wicked cool to be
+	   able to do multiple files, offering to zip them or whatever like
+	   nautilus-sendto does.  Note that text/uri-list is defined to have
+	   each line terminated by \r\n, but we can be tolerant of applications
+	   that only use \n or don't terminate single-line entries.
+	*/
+	nl = strstr (uri_list, "\r\n");
+	if (!nl) {
+		nl = strchr (uri_list, '\n');
+	}
+	if (nl) {
+		gchar *uri = g_strndup (uri_list, nl - uri_list);
+		file = g_file_new_for_uri (uri);
+		g_free (uri);
+	}
+	else {
+		file = g_file_new_for_uri (uri_list);
+	}
+
+	empathy_send_file (contact, file);
+
+	g_object_unref (file);
+}
+
 static void
 file_manager_send_file_response_cb (GtkDialog      *widget,
 				    gint            response_id,
 				    EmpathyContact *contact)
 {
-	EmpathyFTFactory *factory;
 	GFile *file;
-	gchar *uri;
-	GtkRecentManager *manager;
 
 	if (response_id == GTK_RESPONSE_OK) {
 		file = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (widget));
-		uri = g_file_get_uri (file);
 
-		factory = empathy_ft_factory_dup_singleton ();
+		empathy_send_file (contact, file);
 
-		empathy_ft_factory_new_transfer_outgoing (factory, contact,
-		                                          file);
-
-		manager = gtk_recent_manager_get_default ();
-		gtk_recent_manager_add_item (manager, uri);
-
-		g_free (uri);
-		g_object_unref (factory);
 		g_object_unref (file);
 	}
 
@@ -1496,7 +1662,7 @@ empathy_send_file_with_file_chooser (EmpathyContact *contact)
 	gtk_widget_show (button);
 	gtk_dialog_add_action_widget (GTK_DIALOG (widget), button,
 				      GTK_RESPONSE_OK);
-	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gtk_widget_set_can_default (button, TRUE);
 	gtk_dialog_set_default_response (GTK_DIALOG (widget),
 					 GTK_RESPONSE_OK);
 
@@ -1559,3 +1725,108 @@ empathy_receive_file_with_file_chooser (EmpathyFTHandler *handler)
 
 	gtk_widget_show (widget);
 }
+
+void
+empathy_string_parser_substr (const gchar *text,
+			      gssize len,
+			      EmpathyStringParser *parsers,
+			      gpointer user_data)
+{
+	if (parsers != NULL && parsers[0].match_func != NULL) {
+		parsers[0].match_func (text, len,
+				       parsers[0].replace_func, parsers + 1,
+				       user_data);
+	}
+}
+
+void
+empathy_string_match_link (const gchar *text,
+			   gssize len,
+			   EmpathyStringReplace replace_func,
+			   EmpathyStringParser *sub_parsers,
+			   gpointer user_data)
+{
+	GRegex     *uri_regex;
+	GMatchInfo *match_info;
+	gboolean    match;
+	gint        last = 0;
+
+	uri_regex = empathy_uri_regex_dup_singleton ();
+	match = g_regex_match_full (uri_regex, text, len, 0, 0, &match_info, NULL);
+	if (match) {
+		gint s = 0, e = 0;
+
+		do {
+			g_match_info_fetch_pos (match_info, 0, &s, &e);
+
+			if (s > last) {
+				/* Append the text between last link (or the
+				 * start of the message) and this link */
+				empathy_string_parser_substr (text + last,
+							      s - last,
+							      sub_parsers,
+							      user_data);
+			}
+
+			replace_func (text + s, e - s, NULL, user_data);
+
+			last = e;
+		} while (g_match_info_next (match_info, NULL));
+	}
+
+	empathy_string_parser_substr (text + last, len - last,
+				      sub_parsers, user_data);
+
+	g_match_info_free (match_info);
+	g_regex_unref (uri_regex);
+}
+
+void
+empathy_string_match_smiley (const gchar *text,
+			     gssize len,
+			     EmpathyStringReplace replace_func,
+			     EmpathyStringParser *sub_parsers,
+			     gpointer user_data)
+{
+	guint last = 0;
+	EmpathySmileyManager *smiley_manager;
+	GSList *hits, *l;
+
+	smiley_manager = empathy_smiley_manager_dup_singleton ();
+	hits = empathy_smiley_manager_parse_len (smiley_manager, text, len);
+
+	for (l = hits; l; l = l->next) {
+		EmpathySmileyHit *hit = l->data;
+
+		if (hit->start > last) {
+			/* Append the text between last smiley (or the
+			 * start of the message) and this smiley */
+			empathy_string_parser_substr (text + last,
+						      hit->start - last,
+						      sub_parsers, user_data);
+		}
+
+		replace_func (text + hit->start, hit->end - hit->start,
+			      hit, user_data);
+
+		last = hit->end;
+
+		empathy_smiley_hit_free (hit);
+	}
+	g_slist_free (hits);
+	g_object_unref (smiley_manager);
+
+	empathy_string_parser_substr (text + last, len - last,
+				      sub_parsers, user_data);
+}
+
+void
+empathy_string_match_all (const gchar *text,
+			  gssize len,
+			  EmpathyStringReplace replace_func,
+			  EmpathyStringParser *sub_parsers,
+			  gpointer user_data)
+{
+	replace_func (text, len, NULL, user_data);
+}
+
