@@ -31,7 +31,7 @@
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyChatroom)
 typedef struct {
-	McAccount *account;
+	TpAccount *account;
 	gchar     *room;
 	gchar     *name;
 	gboolean   auto_connect;
@@ -84,7 +84,7 @@ empathy_chatroom_class_init (EmpathyChatroomClass *klass)
 					 g_param_spec_object ("account",
 							      "Chatroom Account",
 							      "The account associated with an chatroom",
-							      MC_TYPE_ACCOUNT,
+							      TP_TYPE_ACCOUNT,
 							      G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class,
@@ -312,22 +312,19 @@ chatroom_set_property (GObject      *object,
 }
 
 EmpathyChatroom *
-empathy_chatroom_new (McAccount *account)
+empathy_chatroom_new (TpAccount *account)
 {
-	g_return_val_if_fail (MC_IS_ACCOUNT (account), NULL);
-
 	return g_object_new (EMPATHY_TYPE_CHATROOM,
 			     "account", account,
 			     NULL);
 }
 
 EmpathyChatroom *
-empathy_chatroom_new_full (McAccount   *account,
+empathy_chatroom_new_full (TpAccount *account,
 			  const gchar *room,
 			  const gchar *name,
 			  gboolean     auto_connect)
 {
-	g_return_val_if_fail (MC_IS_ACCOUNT (account), NULL);
 	g_return_val_if_fail (room != NULL, NULL);
 
 	return g_object_new (EMPATHY_TYPE_CHATROOM,
@@ -338,7 +335,7 @@ empathy_chatroom_new_full (McAccount   *account,
 			     NULL);
 }
 
-McAccount *
+TpAccount *
 empathy_chatroom_get_account (EmpathyChatroom *chatroom)
 {
 	EmpathyChatroomPriv *priv;
@@ -351,12 +348,11 @@ empathy_chatroom_get_account (EmpathyChatroom *chatroom)
 
 void
 empathy_chatroom_set_account (EmpathyChatroom *chatroom,
-			     McAccount      *account)
+			     TpAccount *account)
 {
 	EmpathyChatroomPriv *priv;
 
 	g_return_if_fail (EMPATHY_IS_CHATROOM (chatroom));
-	g_return_if_fail (MC_IS_ACCOUNT (account));
 
 	priv = GET_PRIV (chatroom);
 
@@ -407,11 +403,11 @@ empathy_chatroom_get_name (EmpathyChatroom *chatroom)
 	g_return_val_if_fail (EMPATHY_IS_CHATROOM (chatroom), NULL);
 
 	priv = GET_PRIV (chatroom);
-	
+
 	if (EMP_STR_EMPTY (priv->name)) {
 		return priv->room;
 	}
-	
+
 	return priv->name;
 }
 
@@ -470,8 +466,8 @@ gboolean
 empathy_chatroom_equal (gconstpointer v1,
 		       gconstpointer v2)
 {
-	McAccount   *account_a;
-	McAccount   *account_b;
+	TpAccount *account_a;
+	TpAccount *account_b;
 	const gchar *room_a;
 	const gchar *room_b;
 
@@ -484,8 +480,7 @@ empathy_chatroom_equal (gconstpointer v1,
 	room_a = empathy_chatroom_get_room (EMPATHY_CHATROOM (v1));
 	room_b = empathy_chatroom_get_room (EMPATHY_CHATROOM (v2));
 
-	return empathy_account_equal (account_a, account_b) &&
-	       !tp_strdiff (room_a, room_b);
+	return account_a == account_b && !tp_strdiff (room_a, room_b);
 }
 
 EmpathyTpChat *
