@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2; -*- */
 /*
  * Copyright (C) 2007 Collabora Ltd.
  * Copyright (C) 2007 Nokia Corporation
@@ -41,11 +41,17 @@ typedef enum
   EMPATHY_DEBUG_FT = 1 << 7,
   EMPATHY_DEBUG_LOCATION = 1 << 8,
   EMPATHY_DEBUG_OTHER = 1 << 9,
+  EMPATHY_DEBUG_SHARE_DESKTOP = 1 << 10,
+  EMPATHY_DEBUG_CONNECTIVITY = 1 << 11,
+  EMPATHY_DEBUG_IMPORT_MC4_ACCOUNTS = 1 << 11,
+  EMPATHY_DEBUG_TESTS = 1 << 12,
+  EMPATHY_DEBUG_VOIP = 1 << 13,
 } EmpathyDebugFlags;
 
 gboolean empathy_debug_flag_is_set (EmpathyDebugFlags flag);
 void empathy_debug (EmpathyDebugFlags flag, const gchar *format, ...)
     G_GNUC_PRINTF (2, 3);
+void empathy_debug_free (void);
 void empathy_debug_set_flags (const gchar *flags_string);
 G_END_DECLS
 
@@ -88,4 +94,7 @@ G_END_DECLS
 #define DEBUGGING 0
 
 #endif /* !defined (ENABLE_DEBUG) */
+
+#define gabble_debug_free() G_STMT_START { } G_STMT_END
+
 #endif /* defined (DEBUG_FLAG) */
