@@ -1151,7 +1151,7 @@ tp_chat_got_remote_contact_cb (EmpathyTpContactFactory *factory,
 
 	if (error) {
 		DEBUG ("Error: %s", error->message);
-		empathy_tp_chat_close (EMPATHY_TP_CHAT (chat));
+		empathy_tp_chat_leave (EMPATHY_TP_CHAT (chat));
 		return;
 	}
 
@@ -1172,7 +1172,7 @@ tp_chat_got_self_contact_cb (EmpathyTpContactFactory *factory,
 
 	if (error) {
 		DEBUG ("Error: %s", error->message);
-		empathy_tp_chat_close (EMPATHY_TP_CHAT (chat));
+		empathy_tp_chat_leave (EMPATHY_TP_CHAT (chat));
 		return;
 	}
 
@@ -1505,7 +1505,7 @@ empathy_tp_chat_new (TpChannel *channel)
 			     NULL);
 }
 
-void
+static void
 empathy_tp_chat_close (EmpathyTpChat *chat) {
 	EmpathyTpChatPriv *priv = GET_PRIV (chat);
 
@@ -1713,6 +1713,13 @@ empathy_tp_chat_acknowledge_messages (EmpathyTpChat *chat,
 	g_slist_free (msgs);
 }
 
+void
+empathy_tp_chat_acknowledge_all_messages (EmpathyTpChat *chat)
+{
+  empathy_tp_chat_acknowledge_messages (chat,
+    (GSList *) empathy_tp_chat_get_pending_messages (chat));
+}
+
 gboolean
 empathy_tp_chat_password_needed (EmpathyTpChat *self)
 {
@@ -1797,3 +1804,43 @@ empathy_tp_chat_can_add_contact (EmpathyTpChat *self)
 		tp_proxy_has_interface_by_id (priv->channel,
 			TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP);;
 }
+
+static void
+leave_remove_members_cb (TpChannel *proxy,
+			 const GError *error,
+			 gpointer user_data,
+			 GObject *weak_object)
+{
+	EmpathyTpChat *self = user_data;
+
+	if (error == NULL)
+		return;
+
+	DEBUG ("RemoveMembers failed (%s); closing the channel", error->message);
+	empathy_tp_chat_close (self);
+}
+
+void
+empathy_tp_chat_leave (EmpathyTpChat *self)
+{
+	EmpathyTpChatPriv *priv = GET_PRIV (self);
+	TpHandle self_handle;
+	GArray *array;
+
+	if (!tp_proxy_has_interface_by_id (priv->channel,
+		TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP))
+		empathy_tp_chat_close (self);
+
+	self_handle = tp_channel_group_get_self_handle (priv->channel);
+	if (self_handle == 0)
+		/* we are not member of the channel */
+		empathy_tp_chat_close (self);
+
+	array = g_array_sized_new (FALSE, FALSE, sizeof (TpHandle), 1);
+	g_array_insert_val (array, 0, self_handle);
+
+	tp_cli_channel_interface_group_call_remove_members (priv->channel, -1, array,
+		"", leave_remove_members_cb, self, NULL, G_OBJECT (self));
+
+	g_array_free (array, TRUE);
+}
