@@ -33,6 +33,7 @@
 #include <dbus/dbus-glib.h>
 
 #include <telepathy-glib/account-manager.h>
+#include <telepathy-glib/defs.h>
 #include <telepathy-glib/util.h>
 
 #include <libempathy/empathy-utils.h>
@@ -87,6 +88,7 @@ typedef struct {
   GtkWidget *treeview;
 
   GtkWidget *button_add;
+  GtkWidget *button_remove;
   GtkWidget *button_import;
 
   GtkWidget *combobox_protocol;
@@ -131,8 +133,8 @@ typedef struct {
 enum {
   COL_NAME,
   COL_STATUS,
-  COL_ACCOUNT_POINTER,
-  COL_ACCOUNT_SETTINGS_POINTER,
+  COL_ACCOUNT,
+  COL_ACCOUNT_SETTINGS,
   COL_COUNT
 };
 
@@ -173,6 +175,10 @@ static void accounts_dialog_presence_changed_cb (TpAccount *account,
     gchar *status_message,
     EmpathyAccountsDialog *dialog);
 
+static void accounts_dialog_model_selection_changed (
+    GtkTreeSelection *selection,
+    EmpathyAccountsDialog *dialog);
+
 static void
 accounts_dialog_update_name_label (EmpathyAccountsDialog *dialog,
     const gchar *display_name)
@@ -180,7 +186,7 @@ accounts_dialog_update_name_label (EmpathyAccountsDialog *dialog,
   gchar *text;
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
 
-  text = g_markup_printf_escaped ("<big><b>%s</b></big>", display_name);
+  text = g_markup_printf_escaped ("<b>%s</b>", display_name);
   gtk_label_set_markup (GTK_LABEL (priv->label_name), text);
 
   g_free (text);
@@ -192,6 +198,7 @@ accounts_dialog_update_status_infobar (EmpathyAccountsDialog *dialog,
 {
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
   const gchar               *message;
+  gchar                     *message_markup;
   gchar                     *status_message = NULL;
   guint                     status;
   guint                     reason;
@@ -211,8 +218,9 @@ accounts_dialog_update_status_infobar (EmpathyAccountsDialog *dialog,
   if (!gtk_tree_selection_get_selected (selection, &model, &iter))
     return;
 
-  gtk_tree_model_get (model, &iter,
-      COL_ACCOUNT_POINTER, &selected_account, -1);
+  gtk_tree_model_get (model, &iter, COL_ACCOUNT, &selected_account, -1);
+  if (selected_account != NULL)
+    g_object_unref (selected_account);
 
   /* do not update the infobar when the account is not selected */
   if (account != selected_account)
@@ -224,6 +232,19 @@ accounts_dialog_update_status_infobar (EmpathyAccountsDialog *dialog,
       presence = tp_account_get_current_presence (account, NULL, &status_message);
       account_enabled = tp_account_is_enabled (account);
       creating_account = FALSE;
+
+      if (status == TP_CONNECTION_STATUS_CONNECTED &&
+          (presence == TP_CONNECTION_PRESENCE_TYPE_OFFLINE ||
+           presence == TP_CONNECTION_PRESENCE_TYPE_UNSET))
+        /* If presence is Unset (CM doesn't implement SimplePresence) but we
+         * are connected, consider ourself as Available.
+         * We also check Offline because of this MC5 bug: fd.o #26060 */
+        presence = TP_CONNECTION_PRESENCE_TYPE_AVAILABLE;
+
+      /* set presence to offline if account is disabled
+       * (else no icon is shown in infobar)*/
+      if (!account_enabled)
+        presence = TP_CONNECTION_PRESENCE_TYPE_OFFLINE;
     }
   else
     {
@@ -314,7 +335,8 @@ accounts_dialog_update_status_infobar (EmpathyAccountsDialog *dialog,
       gtk_widget_hide (priv->throbber);
     }
 
-  gtk_label_set_text (GTK_LABEL (priv->label_status), message);
+  message_markup = g_markup_printf_escaped ("<i>%s</i>", message);
+  gtk_label_set_markup (GTK_LABEL (priv->label_status), message_markup);
   gtk_widget_show (priv->label_status);
 
   if (!creating_account)
@@ -323,6 +345,7 @@ accounts_dialog_update_status_infobar (EmpathyAccountsDialog *dialog,
     gtk_widget_hide (priv->infobar);
 
   g_free (status_message);
+  g_free (message_markup);
 }
 
 static void
@@ -345,8 +368,8 @@ empathy_account_dialog_widget_cancelled_cb (
     return;
 
   gtk_tree_model_get (model, &iter,
-      COL_ACCOUNT_SETTINGS_POINTER, &settings,
-      COL_ACCOUNT_POINTER, &account, -1);
+      COL_ACCOUNT_SETTINGS, &settings,
+      COL_ACCOUNT, &account, -1);
 
   empathy_account_widget_discard_pending_changes (priv->setting_widget_object);
 
@@ -365,6 +388,7 @@ empathy_account_dialog_widget_cancelled_cb (
 
   gtk_widget_set_sensitive (priv->treeview, TRUE);
   gtk_widget_set_sensitive (priv->button_add, TRUE);
+  gtk_widget_set_sensitive (priv->button_remove, TRUE);
   gtk_widget_set_sensitive (priv->button_import, TRUE);
 
   if (settings != NULL)
@@ -395,6 +419,7 @@ empathy_account_dialog_account_created_cb (EmpathyAccountWidget *widget_object,
 
   gtk_widget_set_sensitive (priv->treeview, TRUE);
   gtk_widget_set_sensitive (priv->button_add, TRUE);
+  gtk_widget_set_sensitive (priv->button_remove, TRUE);
   gtk_widget_set_sensitive (priv->button_import, TRUE);
 
   empathy_signal_connect_weak (account, "status-changed",
@@ -497,7 +522,7 @@ accounts_dialog_has_pending_change (EmpathyAccountsDialog *dialog,
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
 
   if (gtk_tree_selection_get_selected (selection, &model, &iter))
-    gtk_tree_model_get (model, &iter, COL_ACCOUNT_POINTER, account, -1);
+    gtk_tree_model_get (model, &iter, COL_ACCOUNT, account, -1);
 
   return priv->setting_widget_object != NULL
       && empathy_account_widget_contains_pending_changes (
@@ -510,19 +535,25 @@ accounts_dialog_setup_ui_to_add_account (EmpathyAccountsDialog *dialog)
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
   EmpathyAccountSettings *settings;
   gchar *str;
-  const gchar *display_name;
+  const gchar *name, *display_name;
   TpConnectionManager *cm;
   TpConnectionManagerProtocol *proto;
-  gboolean is_gtalk;
+  gboolean is_gtalk, is_facebook;
 
   cm = empathy_protocol_chooser_dup_selected (
-      EMPATHY_PROTOCOL_CHOOSER (priv->combobox_protocol), &proto, &is_gtalk);
+      EMPATHY_PROTOCOL_CHOOSER (priv->combobox_protocol), &proto, &is_gtalk,
+      &is_facebook);
   if (cm == NULL)
     return;
 
-  display_name = empathy_protocol_name_to_display_name (
-      is_gtalk ? "gtalk" : proto->name);
+  if (is_gtalk)
+    name = "gtalk";
+  else if (is_facebook)
+    name ="facebook";
+  else
+    name = proto->name;
 
+  display_name = empathy_protocol_name_to_display_name (name);
   if (display_name == NULL)
     display_name = proto->name;
 
@@ -538,6 +569,9 @@ accounts_dialog_setup_ui_to_add_account (EmpathyAccountsDialog *dialog)
   if (is_gtalk)
     empathy_account_settings_set_icon_name_async (settings, "im-google-talk",
         NULL, NULL);
+  else if (is_facebook)
+    empathy_account_settings_set_icon_name_async (settings, "im-facebook",
+        NULL, NULL);
 
   accounts_dialog_add (dialog, settings);
   accounts_dialog_model_set_selected (dialog, settings);
@@ -557,6 +591,8 @@ accounts_dialog_protocol_changed_cb (GtkWidget *widget,
   GtkTreeModel *model;
   GtkTreeIter iter;
   gboolean creating;
+  EmpathyAccountSettings *settings;
+  gchar *account = NULL, *password = NULL;
 
   /* The "changed" signal is fired during the initiation of the
    * EmpathyProtocolChooser while populating the widget. Such signals should
@@ -574,12 +610,39 @@ accounts_dialog_protocol_changed_cb (GtkWidget *widget,
   if (!gtk_tree_selection_get_selected (selection, &model, &iter))
     return;
 
+  /* Save "account" and "password" parameters */
+  g_object_get (priv->setting_widget_object, "settings", &settings, NULL);
+
+  if (settings != NULL)
+    {
+      account = g_strdup (empathy_account_settings_get_string (settings,
+            "account"));
+      password = g_strdup (empathy_account_settings_get_string (settings,
+            "password"));
+      g_object_unref (settings);
+    }
+
   /* We are creating a new widget to replace the current one, don't ask
    * confirmation to the user. */
   priv->force_change_row = TRUE;
   gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
 
   accounts_dialog_setup_ui_to_add_account (dialog);
+
+  /* Restore "account" and "password" parameters in the new widget */
+  if (account != NULL)
+    {
+      empathy_account_widget_set_account_param (priv->setting_widget_object,
+          account);
+      g_free (account);
+    }
+
+  if (password != NULL)
+    {
+      empathy_account_widget_set_password_param (priv->setting_widget_object,
+          password);
+      g_free (password);
+    }
 }
 
 static void
@@ -683,8 +746,12 @@ accounts_dialog_button_add_clicked_cb (GtkWidget *button,
       accounts_dialog_setup_ui_to_add_account (dialog);
       gtk_widget_set_sensitive (priv->treeview, FALSE);
       gtk_widget_set_sensitive (priv->button_add, FALSE);
+      gtk_widget_set_sensitive (priv->button_remove, FALSE);
       gtk_widget_set_sensitive (priv->button_import, FALSE);
     }
+
+  if (account != NULL)
+    g_object_unref (account);
 }
 
 static void
@@ -777,8 +844,77 @@ accounts_dialog_name_editing_started_cb (GtkCellRenderer *renderer,
   DEBUG ("Editing account name started; stopping flashing");
 }
 
+static const gchar *
+get_status_icon_for_account (EmpathyAccountsDialog *self,
+    TpAccount *account)
+{
+  EmpathyAccountsDialogPriv *priv = GET_PRIV (self);
+  TpConnectionStatus status;
+  TpConnectionStatusReason reason;
+  TpConnectionPresenceType presence;
+
+  if (account == NULL)
+    return empathy_icon_name_for_presence (TP_CONNECTION_PRESENCE_TYPE_OFFLINE);
+
+  if (!tp_account_is_enabled (account))
+    return empathy_icon_name_for_presence (TP_CONNECTION_PRESENCE_TYPE_OFFLINE);
+
+  status = tp_account_get_connection_status (account, &reason);
+
+  if (status == TP_CONNECTION_STATUS_DISCONNECTED)
+    {
+      if (reason != TP_CONNECTION_STATUS_REASON_REQUESTED)
+        /* An error occured */
+        return GTK_STOCK_DIALOG_ERROR;
+
+      presence = TP_CONNECTION_PRESENCE_TYPE_OFFLINE;
+    }
+  else if (status == TP_CONNECTION_STATUS_CONNECTING)
+    {
+      /* Account is connecting. Display a blinking account alternating between
+       * the offline icon and the requested presence. */
+      if (priv->connecting_show)
+        presence = tp_account_get_requested_presence (account, NULL, NULL);
+      else
+        presence = TP_CONNECTION_PRESENCE_TYPE_OFFLINE;
+    }
+  else
+    {
+      /* status == TP_CONNECTION_STATUS_CONNECTED */
+      presence = tp_account_get_current_presence (account, NULL, NULL);
+
+      /* If presence is Unset (CM doesn't implement SimplePresence),
+       * display the 'available' icon.
+       * We also check Offline because of this MC5 bug: fd.o #26060 */
+      if (presence == TP_CONNECTION_PRESENCE_TYPE_OFFLINE ||
+          presence == TP_CONNECTION_PRESENCE_TYPE_UNSET)
+        presence = TP_CONNECTION_PRESENCE_TYPE_AVAILABLE;
+    }
+
+  return empathy_icon_name_for_presence (presence);
+}
+
 static void
-accounts_dialog_model_pixbuf_data_func (GtkTreeViewColumn *tree_column,
+accounts_dialog_model_status_pixbuf_data_func (GtkTreeViewColumn *tree_column,
+    GtkCellRenderer *cell,
+    GtkTreeModel *model,
+    GtkTreeIter *iter,
+    EmpathyAccountsDialog *dialog)
+{
+  TpAccount *account;
+
+  gtk_tree_model_get (model, iter, COL_ACCOUNT, &account, -1);
+
+  g_object_set (cell,
+      "icon-name", get_status_icon_for_account (dialog, account),
+      NULL);
+
+  if (account != NULL)
+    g_object_unref (account);
+}
+
+static void
+accounts_dialog_model_protocol_pixbuf_data_func (GtkTreeViewColumn *tree_column,
     GtkCellRenderer *cell,
     GtkTreeModel *model,
     GtkTreeIter *iter,
@@ -788,39 +924,15 @@ accounts_dialog_model_pixbuf_data_func (GtkTreeViewColumn *tree_column,
   gchar              *icon_name;
   GdkPixbuf          *pixbuf;
   TpConnectionStatus  status;
-  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
 
   gtk_tree_model_get (model, iter,
       COL_STATUS, &status,
-      COL_ACCOUNT_SETTINGS_POINTER, &settings,
+      COL_ACCOUNT_SETTINGS, &settings,
       -1);
 
   icon_name = empathy_account_settings_get_icon_name (settings);
   pixbuf = empathy_pixbuf_from_icon_name (icon_name, GTK_ICON_SIZE_BUTTON);
 
-  if (pixbuf)
-    {
-      if (status == TP_CONNECTION_STATUS_DISCONNECTED ||
-          (status == TP_CONNECTION_STATUS_CONNECTING &&
-              !priv->connecting_show))
-        {
-          GdkPixbuf *modded_pixbuf;
-
-          modded_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
-              TRUE,
-              8,
-              gdk_pixbuf_get_width (pixbuf),
-              gdk_pixbuf_get_height (pixbuf));
-
-          gdk_pixbuf_saturate_and_pixelate (pixbuf,
-              modded_pixbuf,
-              1.0,
-              TRUE);
-          g_object_unref (pixbuf);
-          pixbuf = modded_pixbuf;
-        }
-    }
-
   g_object_set (cell,
       "visible", TRUE,
       "pixbuf", pixbuf,
@@ -886,7 +998,7 @@ accounts_dialog_name_edited_cb (GtkCellRendererText *renderer,
   treepath = gtk_tree_path_new_from_string (path);
   gtk_tree_model_get_iter (model, &iter, treepath);
   gtk_tree_model_get (model, &iter,
-      COL_ACCOUNT_SETTINGS_POINTER, &settings,
+      COL_ACCOUNT_SETTINGS, &settings,
       -1);
   gtk_list_store_set (GTK_LIST_STORE (model), &iter,
       COL_NAME, new_text,
@@ -918,7 +1030,7 @@ accounts_dialog_delete_account_response_cb (GtkDialog *message_dialog,
       if (!gtk_tree_selection_get_selected (selection, &model, &iter))
         return;
 
-      gtk_tree_model_get (model, &iter, COL_ACCOUNT_POINTER, &account, -1);
+      gtk_tree_model_get (model, &iter, COL_ACCOUNT, &account, -1);
 
       if (account != NULL)
         {
@@ -927,7 +1039,17 @@ accounts_dialog_delete_account_response_cb (GtkDialog *message_dialog,
           account = NULL;
         }
 
+      /* No need to call accounts_dialog_model_selection_changed while
+       * removing as we are going to call accounts_dialog_model_select_first
+       * right after which will update the selection. */
+      g_signal_handlers_block_by_func (selection,
+          accounts_dialog_model_selection_changed, account_dialog);
+
       gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+
+      g_signal_handlers_unblock_by_func (selection,
+          accounts_dialog_model_selection_changed, account_dialog);
+
       accounts_dialog_model_select_first (account_dialog);
     }
 
@@ -935,26 +1057,23 @@ accounts_dialog_delete_account_response_cb (GtkDialog *message_dialog,
 }
 
 static void
-accounts_dialog_view_delete_activated_cb (EmpathyCellRendererActivatable *cell,
-    const gchar *path_string,
-    EmpathyAccountsDialog *dialog)
+accounts_dialog_remove_account_iter (EmpathyAccountsDialog *dialog,
+    GtkTreeIter *iter)
 {
+  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
   TpAccount *account;
   GtkTreeModel *model;
-  GtkTreeIter iter;
   gchar *question_dialog_primary_text;
-  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
 
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
 
-  if (!gtk_tree_model_get_iter_from_string (model, &iter, path_string))
-    return;
-
-  gtk_tree_model_get (model, &iter, COL_ACCOUNT_POINTER, &account, -1);
+  gtk_tree_model_get (model, iter, COL_ACCOUNT, &account, -1);
 
   if (account == NULL || !tp_account_is_valid (account))
     {
-      gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+      if (account != NULL)
+        g_object_unref (account);
+      gtk_list_store_remove (GTK_LIST_STORE (model), iter);
       accounts_dialog_model_select_first (dialog);
       return;
     }
@@ -971,14 +1090,46 @@ accounts_dialog_view_delete_activated_cb (EmpathyCellRendererActivatable *cell,
       GTK_STOCK_REMOVE, GTK_RESPONSE_YES, NULL);
 
   g_free (question_dialog_primary_text);
-
-  if (account != NULL)
-    {
-      g_object_unref (account);
-      account = NULL;
-    }
+  g_object_unref (account);
 }
 
+static void
+accounts_dialog_button_remove_clicked_cb (GtkWidget *button,
+    EmpathyAccountsDialog *dialog)
+{
+  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
+  GtkTreeView  *view;
+  GtkTreeModel *model;
+  GtkTreeSelection *selection;
+  GtkTreeIter iter;
+
+  view = GTK_TREE_VIEW (priv->treeview);
+  model = gtk_tree_view_get_model (view);
+  selection = gtk_tree_view_get_selection (view);
+  if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
+      return;
+
+  accounts_dialog_remove_account_iter (dialog, &iter);
+}
+
+#ifdef HAVE_MEEGO
+static void
+accounts_dialog_view_delete_activated_cb (EmpathyCellRendererActivatable *cell,
+    const gchar *path_string,
+    EmpathyAccountsDialog *dialog)
+{
+  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+
+  model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
+  if (!gtk_tree_model_get_iter_from_string (model, &iter, path_string))
+    return;
+
+  accounts_dialog_remove_account_iter (dialog, &iter);
+}
+#endif /* HAVE_MEEGO */
+
 static void
 accounts_dialog_model_add_columns (EmpathyAccountsDialog *dialog)
 {
@@ -995,12 +1146,21 @@ accounts_dialog_model_add_columns (EmpathyAccountsDialog *dialog)
   gtk_tree_view_column_set_expand (column, TRUE);
   gtk_tree_view_append_column (view, column);
 
-  /* Icon renderer */
+  /* Status icon renderer */
   cell = gtk_cell_renderer_pixbuf_new ();
   gtk_tree_view_column_pack_start (column, cell, FALSE);
   gtk_tree_view_column_set_cell_data_func (column, cell,
       (GtkTreeCellDataFunc)
-      accounts_dialog_model_pixbuf_data_func,
+      accounts_dialog_model_status_pixbuf_data_func,
+      dialog,
+      NULL);
+
+  /* Protocol icon renderer */
+  cell = gtk_cell_renderer_pixbuf_new ();
+  gtk_tree_view_column_pack_start (column, cell, FALSE);
+  gtk_tree_view_column_set_cell_data_func (column, cell,
+      (GtkTreeCellDataFunc)
+      accounts_dialog_model_protocol_pixbuf_data_func,
       dialog,
       NULL);
 
@@ -1021,6 +1181,7 @@ accounts_dialog_model_add_columns (EmpathyAccountsDialog *dialog)
       dialog);
   g_object_set (cell, "ypad", 4, NULL);
 
+#ifdef HAVE_MEEGO
   /* Delete column */
   cell = empathy_cell_renderer_activatable_new ();
   gtk_tree_view_column_pack_start (column, cell, FALSE);
@@ -1032,6 +1193,7 @@ accounts_dialog_model_add_columns (EmpathyAccountsDialog *dialog)
   g_signal_connect (cell, "path-activated",
       G_CALLBACK (accounts_dialog_view_delete_activated_cb),
       dialog);
+#endif /* HAVE_MEEGO */
 }
 
 static EmpathyAccountSettings *
@@ -1051,7 +1213,7 @@ accounts_dialog_model_get_selected_settings (EmpathyAccountsDialog *dialog)
     return NULL;
 
   gtk_tree_model_get (model, &iter,
-      COL_ACCOUNT_SETTINGS_POINTER, &settings, -1);
+      COL_ACCOUNT_SETTINGS, &settings, -1);
 
   return settings;
 }
@@ -1060,6 +1222,7 @@ static void
 accounts_dialog_model_selection_changed (GtkTreeSelection *selection,
     EmpathyAccountsDialog *dialog)
 {
+  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
   EmpathyAccountSettings *settings;
   GtkTreeModel *model;
   GtkTreeIter   iter;
@@ -1072,6 +1235,9 @@ accounts_dialog_model_selection_changed (GtkTreeSelection *selection,
 
   if (settings != NULL)
     g_object_unref (settings);
+
+  /* Update remove button sensitivity */
+  gtk_widget_set_sensitive (priv->button_remove, is_selection);
 }
 
 static void
@@ -1126,6 +1292,7 @@ accounts_dialog_account_selection_change (GtkTreeSelection *selection,
   TpAccount *account = NULL;
   EmpathyAccountsDialog *dialog = EMPATHY_ACCOUNTS_DIALOG (data);
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
+  gboolean ret;
 
   if (priv->force_change_row)
     {
@@ -1154,13 +1321,17 @@ accounts_dialog_account_selection_change (GtkTreeSelection *selection,
           GTK_STOCK_DISCARD, GTK_RESPONSE_YES, NULL);
 
       g_free (question_dialog_primary_text);
+      ret = FALSE;
     }
   else
     {
-      return TRUE;
+      ret = TRUE;
     }
 
-  return FALSE;
+  if (account != NULL)
+    g_object_unref (account);
+
+  return ret;
 }
 
 static void
@@ -1220,7 +1391,7 @@ accounts_dialog_get_settings_iter (EmpathyAccountsDialog *dialog,
       gboolean   equal;
 
       gtk_tree_model_get (model, iter,
-          COL_ACCOUNT_SETTINGS_POINTER, &this_settings,
+          COL_ACCOUNT_SETTINGS, &this_settings,
           -1);
 
       equal = (this_settings == settings);
@@ -1257,7 +1428,7 @@ accounts_dialog_get_account_iter (EmpathyAccountsDialog *dialog,
       gboolean   equal;
 
       gtk_tree_model_get (model, iter,
-          COL_ACCOUNT_SETTINGS_POINTER, &settings,
+          COL_ACCOUNT_SETTINGS, &settings,
           -1);
 
       equal = empathy_account_settings_has_account (settings, account);
@@ -1299,7 +1470,7 @@ accounts_dialog_add (EmpathyAccountsDialog *dialog,
   gtk_list_store_set (GTK_LIST_STORE (model), &iter,
       COL_NAME, name,
       COL_STATUS, TP_CONNECTION_STATUS_DISCONNECTED,
-      COL_ACCOUNT_SETTINGS_POINTER, settings,
+      COL_ACCOUNT_SETTINGS, settings,
       -1);
 }
 
@@ -1350,6 +1521,25 @@ accounts_dialog_connection_changed_cb (TpAccount *account,
         dialog);
 }
 
+static void
+update_account_in_treeview (EmpathyAccountsDialog *self,
+    TpAccount *account)
+{
+  EmpathyAccountsDialogPriv *priv = GET_PRIV (self);
+  GtkTreeIter iter;
+  GtkTreeModel *model;
+
+  model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
+  if (accounts_dialog_get_account_iter (self, account, &iter))
+    {
+      GtkTreePath *path;
+
+      path = gtk_tree_model_get_path (model, &iter);
+      gtk_tree_model_row_changed (model, path, &iter);
+      gtk_tree_path_free (path);
+    }
+}
+
 static void
 accounts_dialog_presence_changed_cb (TpAccount *account,
     guint presence,
@@ -1359,6 +1549,8 @@ accounts_dialog_presence_changed_cb (TpAccount *account,
 {
   /* Update the status-infobar in the details view */
   accounts_dialog_update_status_infobar (dialog, account);
+
+  update_account_in_treeview (dialog, account);
 }
 
 static void
@@ -1420,8 +1612,8 @@ accounts_dialog_add_account (EmpathyAccountsDialog *dialog,
   gtk_list_store_set (GTK_LIST_STORE (model), &iter,
       COL_NAME, name,
       COL_STATUS, status,
-      COL_ACCOUNT_POINTER, account,
-      COL_ACCOUNT_SETTINGS_POINTER, settings,
+      COL_ACCOUNT, account,
+      COL_ACCOUNT_SETTINGS, settings,
       -1);
 
   accounts_dialog_connection_changed_cb (account,
@@ -1508,6 +1700,7 @@ accounts_dialog_account_disabled_cb (TpAccountManager *manager,
     EmpathyAccountsDialog *dialog)
 {
   enable_or_disable_account (dialog, account, FALSE);
+  update_account_in_treeview (dialog, account);
 }
 
 static void
@@ -1565,23 +1758,24 @@ accounts_dialog_set_selected_account (EmpathyAccountsDialog *dialog,
 }
 
 static void
-accounts_dialog_cms_ready_cb (EmpathyConnectionManagers *cms,
-    GParamSpec *pspec,
-    EmpathyAccountsDialog *dialog)
+accounts_dialog_cms_prepare_cb (GObject *source,
+    GAsyncResult *result,
+    gpointer user_data)
 {
+  EmpathyConnectionManagers *cms = EMPATHY_CONNECTION_MANAGERS (source);
+  EmpathyAccountsDialog *dialog = user_data;
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
 
-  if (empathy_connection_managers_is_ready (cms))
-    {
-      accounts_dialog_update_settings (dialog, NULL);
+  if (!empathy_connection_managers_prepare_finish (cms, result, NULL))
+    return;
 
-      if (priv->initial_selection != NULL)
-        {
-          accounts_dialog_set_selected_account
-              (dialog, priv->initial_selection);
-          g_object_unref (priv->initial_selection);
-          priv->initial_selection = NULL;
-        }
+  accounts_dialog_update_settings (dialog, NULL);
+
+  if (priv->initial_selection != NULL)
+    {
+      accounts_dialog_set_selected_account (dialog, priv->initial_selection);
+      g_object_unref (priv->initial_selection);
+      priv->initial_selection = NULL;
     }
 }
 
@@ -1618,9 +1812,9 @@ accounts_dialog_accounts_setup (EmpathyAccountsDialog *dialog)
   g_list_free (accounts);
 
   priv->cms = empathy_connection_managers_dup_singleton ();
-  if (!empathy_connection_managers_is_ready (priv->cms))
-    g_signal_connect (priv->cms, "notify::ready",
-        G_CALLBACK (accounts_dialog_cms_ready_cb), dialog);
+
+  empathy_connection_managers_prepare_async (priv->cms,
+      accounts_dialog_cms_prepare_cb, dialog);
 
   accounts_dialog_model_select_first (dialog);
 }
@@ -1679,6 +1873,9 @@ dialog_response_cb (GtkWidget *widget,
         {
           gtk_widget_destroy (widget);
         }
+
+      if (account != NULL)
+        g_object_unref (account);
     }
 }
 
@@ -1690,7 +1887,7 @@ accounts_dialog_build_ui (EmpathyAccountsDialog *dialog)
   gchar                        *filename;
   EmpathyAccountsDialogPriv    *priv = GET_PRIV (dialog);
   GtkWidget                    *content_area;
-  GtkWidget *action_area;
+  GtkWidget *action_area, *vbox, *hbox, *align;
 
   filename = empathy_file_lookup ("empathy-accounts-dialog.ui", "src");
 
@@ -1701,16 +1898,18 @@ accounts_dialog_build_ui (EmpathyAccountsDialog *dialog)
       "alignment_settings", &priv->alignment_settings,
       "alignment_infobar", &priv->alignment_infobar,
       "treeview", &priv->treeview,
-      "image_type", &priv->image_type,
-      "label_name", &priv->label_name,
       "button_add", &priv->button_add,
+      "button_remove", &priv->button_remove,
       "button_import", &priv->button_import,
       "hbox_protocol", &priv->hbox_protocol,
       NULL);
   g_free (filename);
 
+  gtk_widget_set_no_show_all (priv->frame_no_protocol, TRUE);
+
   empathy_builder_connect (gui, dialog,
       "button_add", "clicked", accounts_dialog_button_add_clicked_cb,
+      "button_remove", "clicked", accounts_dialog_button_remove_clicked_cb,
       "button_import", "clicked", accounts_dialog_button_import_clicked_cb,
       NULL);
 
@@ -1722,9 +1921,13 @@ accounts_dialog_build_ui (EmpathyAccountsDialog *dialog)
 
   action_area = gtk_dialog_get_action_area (GTK_DIALOG (dialog));
 
-#ifdef HAVE_MOBLIN
+#ifdef HAVE_MEEGO
   gtk_widget_hide (action_area);
-#endif
+  gtk_widget_hide (priv->button_remove);
+#endif /* HAVE_MEEGO */
+
+  /* Remove button is unsensitive until we have a selected account */
+  gtk_widget_set_sensitive (priv->button_remove, FALSE);
 
   priv->combobox_protocol = empathy_protocol_chooser_new ();
   gtk_box_pack_start (GTK_BOX (priv->hbox_protocol), priv->combobox_protocol,
@@ -1737,27 +1940,57 @@ accounts_dialog_build_ui (EmpathyAccountsDialog *dialog)
     gtk_window_set_transient_for (GTK_WINDOW (dialog),
         priv->parent_window);
 
-  /* set up spinner */
-  priv->throbber = ephy_spinner_new ();
-  ephy_spinner_set_size (EPHY_SPINNER (priv->throbber), GTK_ICON_SIZE_SMALL_TOOLBAR);
-
   priv->infobar = gtk_info_bar_new ();
   gtk_container_add (GTK_CONTAINER (priv->alignment_infobar),
       priv->infobar);
   gtk_widget_show (priv->infobar);
 
+  content_area = gtk_info_bar_get_content_area (GTK_INFO_BAR (priv->infobar));
+
+  priv->image_type = gtk_image_new_from_stock (GTK_STOCK_CUT,
+      GTK_ICON_SIZE_DIALOG);
+  gtk_misc_set_alignment (GTK_MISC (priv->image_type), 0.0, 0.5);
+  gtk_box_pack_start (GTK_BOX (content_area), priv->image_type, FALSE, FALSE, 0);
+  gtk_widget_show (priv->image_type);
+
+  vbox = gtk_vbox_new (FALSE, 6);
+  gtk_box_pack_start (GTK_BOX (content_area), vbox, TRUE, TRUE, 0);
+  gtk_widget_show (vbox);
+
+  /* first row */
+  align = gtk_alignment_new (0.5, 0.0, 0.0, 0.0);
+  gtk_widget_show (align);
+
+  priv->label_name = gtk_label_new (NULL);
+  gtk_container_add (GTK_CONTAINER (align), priv->label_name);
+  gtk_widget_show (priv->label_name);
+
+  gtk_box_pack_start (GTK_BOX (vbox), align, TRUE, TRUE, 0);
+
+  /* second row */
+  align = gtk_alignment_new (0.5, 0.0, 0.0, 0.0);
+  gtk_widget_show (align);
+  hbox = gtk_hbox_new (FALSE, 6);
+  gtk_widget_show (hbox);
+  gtk_container_add (GTK_CONTAINER (align), hbox);
+
+  gtk_box_pack_start (GTK_BOX (vbox), align, TRUE, TRUE, 0);
+
+  /* set up spinner */
+  priv->throbber = ephy_spinner_new ();
+  ephy_spinner_set_size (EPHY_SPINNER (priv->throbber), GTK_ICON_SIZE_SMALL_TOOLBAR);
+
   priv->image_status = gtk_image_new_from_icon_name (
             empathy_icon_name_for_presence (
             TP_CONNECTION_PRESENCE_TYPE_OFFLINE), GTK_ICON_SIZE_SMALL_TOOLBAR);
 
   priv->label_status = gtk_label_new (NULL);
+  gtk_label_set_line_wrap (GTK_LABEL (priv->label_status), TRUE);
+  gtk_widget_show (priv->label_status);
 
-  content_area = gtk_info_bar_get_content_area (GTK_INFO_BAR (priv->infobar));
-  gtk_box_pack_start (GTK_BOX (content_area), priv->throbber,
-      FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (content_area), priv->image_status,
-      FALSE, FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (content_area), priv->label_status);
+  gtk_box_pack_start (GTK_BOX (hbox), priv->throbber, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), priv->image_status, FALSE, FALSE, 3);
+  gtk_box_pack_start (GTK_BOX (hbox), priv->label_status, TRUE, TRUE, 0);
 
   /* Tweak the dialog */
   gtk_window_set_title (GTK_WINDOW (dialog), _("Accounts"));
@@ -1991,3 +2224,68 @@ empathy_accounts_dialog_show (GtkWindow *parent,
 
   return GTK_WIDGET (dialog);
 }
+
+void
+empathy_accounts_dialog_show_application (GdkScreen *screen,
+    GChildWatchFunc application_exit_cb,
+    gpointer user_data,
+    TpAccount *selected_account,
+    gboolean try_import,
+    gboolean hidden)
+{
+  gint command_pid;
+  GError *error = NULL;
+  gchar *argv[4] = { NULL, };
+  gint i = 0;
+  gchar *account_option = NULL;
+  gchar *path;
+
+  g_return_if_fail (GDK_IS_SCREEN (screen));
+  g_return_if_fail (!selected_account || TP_IS_ACCOUNT (selected_account));
+
+  /* Try to run from source directory if possible */
+  path = g_build_filename (g_getenv ("EMPATHY_SRCDIR"), "src",
+      "empathy-accounts", NULL);
+
+  if (!g_file_test (path, G_FILE_TEST_EXISTS))
+    {
+      g_free (path);
+      path = g_build_filename (BIN_DIR, "empathy-accounts", NULL);
+    }
+
+  argv[i++] = path;
+
+  if (selected_account)
+    {
+      const gchar *account_path;
+
+      account_path = tp_proxy_get_object_path (TP_PROXY (selected_account));
+      account_option = g_strdup_printf ("--select-account=%s",
+          &account_path[strlen (TP_ACCOUNT_OBJECT_PATH_BASE)]);
+
+      argv[i++] = account_option;
+    }
+
+  if (try_import)
+    argv[i++] = "--import";
+
+  if (hidden)
+    argv[i++] = "--hidden";
+
+  gdk_spawn_on_screen (screen, NULL, argv, NULL,
+      G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
+      &command_pid, &error);
+  if (error)
+    {
+      g_warning ("Failed to open accounts dialog: %s", error->message);
+      g_error_free (error);
+    }
+
+  /* XXX: unportable cast to GPid; then again, gdk_spawn_on_screen() seems
+   * unportable since it always takes a gint* for the PID */
+  if (application_exit_cb)
+    g_child_watch_add ((GPid) command_pid, application_exit_cb, NULL);
+
+  g_free (account_option);
+  g_free (path);
+}
